/*
 *      nlconnections.h
 *      
 *      Copyright 2012 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __NLCONNECTIONS_H__
#define __NLCONNECTIONS_H__

#include <time.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <linux/netlink.h>
#include <linux/inet_diag.h>

struct _nlconnections_socket {
	guint inode;
	guint dqueue;
	uid_t userid;
	time_t updatetime;
	pid_t apppid;
	gchar *appname;
	gchar *dsthostname;
	gchar srcaddr[INET6_ADDRSTRLEN + 8];
	gchar dstaddr[INET6_ADDRSTRLEN + 8];
	gushort srcport;
	guchar state;
	guchar family;
};

typedef struct _nlconnections_socket *nlconnections_socket_t;

struct socket_request {
	struct nlmsghdr nlhdr;
	struct inet_diag_req nlreq;
};

struct _nlconnections_data {
	gint nlfd;
	pid_t userid;
	GHashTable *connections;
	struct socket_request request;
	struct iovec reqiov;
	struct sockaddr_nl nladdr;
	struct msghdr message;
};

typedef struct _nlconnections_data *nlconnections_data_t;

gboolean nlconnections_terminate_application(pid_t pid);
gchar *nlconnections_socket_state(guchar state);
gboolean nlconnections_update(nlconnections_data_t data);
void nlconnections_close(nlconnections_data_t data);
nlconnections_data_t nlconnections_open(void);

#endif /* __NLCONNECTIONS_H__ */
